from ._thanker import Thanker


__version__ = "0.0.3"
__url__ = "https://github.com/WardPearce/thanker"
__description__ = "Don't be a wanker, be a thanker!"
__author__ = "WardPearce"
__author_email__ = "wardpearce@protonmail.com"
__license__ = " Apache-2.0"


__all__ = [
    "Thanker"
]



import asynctest

from . import Thanker


class TestThanker(asynctest.TestCase):
    use_default_loop = True

    async def setUp(self) -> None:
        self.thanker = Thanker(["thanker"])

    async def test_style(self) -> None:
        async with self.thanker as thanks:
            self.assertIsInstance(await thanks.style(), str)

    async def test_package_info(self) -> None:
        async with self.thanker as thinks:
            self.assertIsInstance(
                await thinks.package_info("aiohttp"),
                dict
            )

    async def test_scan(self) -> None:
        async with self.thanker as thinks:
            async for info in thinks.scan("aiohttp"):
                self.assertIsInstance(info, dict)



import click
import asyncio

from typing import Optional
from thanker import Thanker

loop = asyncio.get_event_loop()


@click.command()
@click.option("--packages", type=str,
              help="List of packages to thank, seperated by a comma")
@click.option("--gratitude_level", type=int, default=None,
              help="Basically the depth of requirements we should go to")
@click.option("--layout", type=str,
              default="- [{name}]({package_url}) by {author}",
              help=" The layout of the thanks, can be any pypi info parameter")
@click.option("--display", default=True, type=bool,
              help="If the output should be displayed in console.")
@click.option("--save", type=click.File(mode="w+"), default=None,
              help="File to save thanks to")
def cli(packages: str, gratitude_level: Optional[int], layout: str,
        display: bool, save) -> None:
    thanker = Thanker(
        packages=packages.split(","),
        gratitude_level=gratitude_level
    )

    async def run_thanks() -> str:
        async with thanker as thanks:
            return await thanks.style(layout=layout)

    thanks_to = loop.run_until_complete(run_thanks())
    if display:
        click.echo(thanks_to)
    if save:
        save.write(thanks_to)



import asyncio

from typing import AsyncGenerator, List, Optional
from aiohttp import ClientSession

from ._scanner import Scanner


class Thanker:
    __pypi_api = "https://pypi.python.org/pypi/{name}/json"

    def __init__(self, packages: List[str],
                 gratitude_level: Optional[int] = None) -> None:
        """Used to thank developers of python packages.

        Parameters
        ----------
        packages : List[str]
            List of root packages.
        gratitude_level : Optional[int], optional
            Basically the depth of requirements we should go to,
            by default None
        """

        self._packages = packages
        self._gratitude_level = gratitude_level

    async def __aenter__(self) -> "Thanker":
        self._requests = ClientSession()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        await self._requests.close()

    async def package_info(self, name: str) -> Optional[dict]:
        """Get information about a package.

        Parameters
        ----------
        name : str

        Returns
        -------
        Optional[dict]
        """

        url = self.__pypi_api.format(name=name)
        async with self._requests.get(url) as resp:
            if resp.status == 200:
                return (await resp.json())["info"]

    async def scan_all(self) -> AsyncGenerator[dict, None]:
        """Scans all packages given.

        Yields
        -------
        dict
        """

        async def convert_to_list(requirement: str) -> List[dict]:
            infos = []
            async for info_ in self.scan(requirement):
                infos.append(info_)
            return infos

        results = await asyncio.gather(*[
            convert_to_list(package) for package in self._packages
        ])

        for result in results:
            for info_ in result:
                yield info_

    async def scan(self, name: str) -> AsyncGenerator[dict, None]:
        """Scan for all requirements of a package .

        Parameters
        ----------
        name : str

        Yields
        -------
        dict
        """

        async for info in Scanner(self).scan(name, self._gratitude_level):
            yield info

    async def style(self, layout: str = "- [{name}]({package_url}) by {author}") -> str:  # noqa: E501
        """Return a string containing all the packages in this package.

        Parameters
        ----------
        layout : str, optional
            The layout of the thanks, can be any pypi info parameter,
            e.g.
            https://github.com/WardPearce/thanker#supported-pypi-parameters,
            by default "- [{name}]({package_url}) by {author}"

        Returns
        -------
        str
        """

        text = ""
        async for info in self.scan_all():
            text += layout.format_map(info) + "\n"

        return text



import re
import asyncio

from typing import TYPE_CHECKING, AsyncGenerator, List, Optional

if TYPE_CHECKING:
    from . import Thanker


class Scanner:
    def __init__(self, upper: "Thanker") -> None:
        self._upper = upper
        self.__ignore = []

    async def scan(self, name: str, current_gratitude: Optional[int] = None,
                   ) -> AsyncGenerator[dict, None]:
        new_gratitude = (
            current_gratitude - 1 if current_gratitude is not None
            else None
        )

        name = (re.sub("[^a-zA-Z\\-_]+", "", name)).lower()
        if new_gratitude != -1 and name not in self.__ignore:
            self.__ignore.append(name)

            info = await self._upper.package_info(name)
            if info:
                yield info

                if info["requires_dist"]:
                    async def convert_to_list(requirement: str) -> List[dict]:
                        infos = []
                        async for info_ in self.scan(requirement,
                                                     new_gratitude):
                            infos.append(info_)
                        return infos

                    results = await asyncio.gather(*[
                        convert_to_list(requirement)
                        for requirement in info["requires_dist"]
                    ])

                    for result in results:
                        for info_ in result:
                            yield info_
